<?php
function buchungValid($buchung_id) {
	// Prüfen ob eine Buchung existiert und zu einem Konto des aktuellen Users gehört
	$sql_b = "SELECT * FROM *PREFIX*md_fibu_buchungen WHERE id=? AND konto IN (SELECT id FROM *PREFIX*md_fibu_konten WHERE uid=?) LIMIT 0,1";
	$res_b = query($sql_b, array($buchung_id, getUID()));
	if($res_b->rowCount() != 1) {
		return false;
	}
	return true;
}

function saveBuchung($konto, $name, $datum, $zeit, $betrag, $description, $kategorie, $is_share_buchung = false) {

	// Prüfen ob alle notwendigen Parameter mit gültigen Werten vorliegen
	if(empty($konto) || !is_numeric($konto)) {
		return array("status" => "error", "message" => "Ungültige Konto-ID: " . $konto);
	}
	if(empty($name)) {
		return array("status" => "error", "message" => "Name fehlt");
	}
	if(empty($datum) || !dateValid($datum)) {
		return array("status" => "error", "message" => "Datum fehlt oder ist ungültig: " . $datum);
	}
	if(empty($zeit) || !timeValid($zeit)) {
		return array("status" => "error", "message" => "Zeit fehlt oder ist ungültig: " . $zeit);
	}
	if(empty($betrag) || !is_numeric($betrag)) {
		return array("status" => "error", "message" => "Ungültiger Betrag: " . $betrag);
	}
	if(!empty($kategorie) && !is_numeric($kategorie)) {
		return array("status" => "error", "message" => "Ungültige Kategorie-ID: " . $kategorie);
	}
	
	// Fehlerhafte Eingaben abfangen und maskieren
	$konto = floor($konto);
	$name = $name;
	$datum = $datum;
	$zeit = $zeit;
	$description = empty($description) ? "" : $description;
	
	// Bei einer Ausleihe-Buchung wird nie eine Kategorie gespeichert
	if($is_share_buchung) {
		$kategorie = 0;
	} else {
		// Prüfen ob die Kategorie existiert und zu dem User gehört
		if(!empty($kategorie) && is_numeric($kategorie) && $kategorie != 0) {
			if(!kategorieValid($kategorie)) {
				return array("status" => "error", "message" => "Kategorie existiert nicht oder gehört zu einem anderen User");
			}
		}
	}
	
	// Wenn wir eine Buchung auf einem eigenen Konto anlegen, prüfen ob uns das Konto gehört
	if(!$is_share_buchung) {
		if(!kontoValid($konto)) {
			return array("status" => "error", "message" => "Konto existiert nicht oder gehört zu einem anderen User");
		}
	}
	
	// Buchung speichern
	$sql_I = "INSERT INTO *PREFIX*md_fibu_buchungen (`konto`, `name`, `datum`, `zeit`, `betrag`, `description`, `kategorie`, `created_by`) VALUES (?, ?, ?, ?, ?, ?, ?, ?);";
	$res_I = query($sql_I, array($konto, $name, $datum, $zeit, $betrag * 100, $description, $kategorie, getUID()));
	if(isSqlError($res_I)) {
		return array("status" => "error", "message" => "Fehler beim Speichern der Buchung: " . getSqlError());
	}
	// ID merken
	$id = sqlInsertId();
	
	// Wenn wir eine Buchung auf einem eigenen Konto anlegen, prüfen ob es ein Ausleihe-Konto ist und dann ggf. dort die Spiegel-Buchung anlegen
	if(!$is_share_buchung) {
		$sql_cl = "SELECT * FROM *PREFIX*md_fibu_share WHERE konto_1=? OR konto_2=? LIMIT 0,1";
		$res_cl = query($sql_cl, array($konto, $konto));
		if($res_cl->rowCount() == 1) {
			$share = $res_cl->fetchRow();
			$share_konto = ($share["konto_1"] == $konto) ? $share["konto_2"] : $share["konto_1"];
			$share_betrag = $share["inverted"] ? $betrag * -1 : $betrag;
			$result = saveBuchung($share_konto, $name, $datum, $zeit, $share_betrag, $description, 0, true);
			if($result["status"] == "error") {
				return array("status" => "error", "message" => "Fehler bei Anlage der Buchung auf Ausleihe-Konto: ".$result["message"]);
			}
			
			// Verbindung merken
			$sql_is = "INSERT INTO *PREFIX*md_fibu_share_connection (`buchung_1`, `buchung_2`) VALUES (?, ?)";
			$res_is = query($sql_is, array($id, $result["id"]));
			if(isSqlError($res_is)) {
				return array("status" => "error", "message" => "Fehler beim Speichern der Share-Verbindung: " . getSqlError());
			}
		}
	}

	// Rückgabe alles OK
	return array("status" => "success", "message" => "", "id" => $id);
}

function editBuchung($id, $name, $datum, $zeit, $betrag, $description, $kategorie, $edit_connected = true) {

	// Übergaben prüfen
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Buchungs-ID übergeben: " . $id);
	} elseif(empty($name)) {
		return array("status" => "error", "message" => "Kein Name übergeben");
	} elseif(empty($betrag) || !is_numeric(str_replace(",", ".", $betrag))) {
		return array("status" => "error", "message" => "Kein Betrag übergeben oder ungültig: " . $betrag);
	} elseif(empty($datum) || !dateValid($datum)) {
		return array("status" => "error", "message" => "Kein Datum übergeben oder ungültig: " . $datum);
	} elseif(empty($zeit) || !timeValid($zeit)) {
		return array("status" => "error", "message" => "Keine Uhrzeit übergeben oder ungültig: " . $zeit);
	} elseif(!empty($kategorie) && !is_numeric($kategorie)) {
		return array("status" => "error", "message" => "Ungültige Kategorie übergeben: " . $kategorie);
	}

	$id = floor($id);
	$betrag = str_replace(",", ".", $betrag);
	
	if($edit_connected) {
		// Prüfen ob die Buchung existiert und zu dem User gehört
		if(!buchungValid($id)) {
			return array("status" => "error", "message" => "Diese Buchung existiert nicht oder gehört nicht zu diesem User");
		}
	}
	
	if($edit_connected) {
		// Prüfen ob wir auf einem geteilten Konto sind
		$sql_cs = "SELECT sc.buchung_1, sc.buchung_2, s.inverted";
		$sql_cs .= " FROM *PREFIX*md_fibu_share_connection sc, *PREFIX*md_fibu_share s, *PREFIX*md_fibu_konten k, *PREFIX*md_fibu_buchungen b";
		$sql_cs .= " WHERE b.id=? AND k.id=b.konto AND (s.konto_1=k.id OR s.konto_2=k.id) AND (sc.buchung_1=b.id OR sc.buchung_2=b.id)";
		$sql_cs .= " LIMIT 0,1";
		$res_cs = query($sql_cs, array($id));
		if($res_cs->rowCount() > 0) {
			$share = $res_cs->fetchRow();
			$id2 = ($share['buchung_1'] == $id) ? $share['buchung_2'] : $share['buchung_1'];
			$betrag2 = ($share['inverted'] == 1) ? $betrag * -1 : $betrag;
			$result2 = editBuchung($id2, $name, $datum, $zeit, $betrag2, $description, 0, false);
			if($result2["status"] == "error") {
				return array("status" => "error", "message" => "Fehler beim Ändern der Share-Buchung: " . $result2["message"]);
			}
		}
	}
	
	// Buchungsdeatils laden
	$sql_b = "SELECT * FROM *PREFIX*md_fibu_buchungen WHERE id='".$id."' LIMIT 0,1";
	$res_b = query($sql_b);
	$buchung = $res_b->fetchRow();
	
	$newVals = "";
	$args = array();
	
	// Betrag prüfen
	if($buchung["betrag"] != $betrag * 100) {
		// Liegt die Buchung in einem anderen Monat?
		if(getMonthFromDate($buchung["datum"]) != getThisMonth()) {
			// Evtl abgeschlossene Monate seit der Buchung löschen damit sie neu berechnet werden
			$sql_dm = "DELETE FROM *PREFIX*md_fibu_months WHERE konto='".$buchung["konto"]."' AND month>='".getMonthFromDate($buchung["datum"])."'";
			$res_dm = query($sql_dm);
			if(isSqlError($res_dm)) {
				return array("status" => "error", "message" => "Fehler beim Auflösen des abgeschlossenen Monats: " . getSqlError());
			}
		}
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "betrag=?";
		$args[] = $betrag * 100;
		
		// evtl. Connection löschen
		$result = deleteConnection($buchung['id'], $edit_connected);
		if($result["status"] == "error") {
			return array("status" => "error", "message" => "Fehler beim Löschen des Verweises: " . $result["message"]);
		}
	}
	
	// Name prüfen
	if($buchung["name"] != $name) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "name=?";
		$args[] = $name;
	}
	
	// Beschreibung prüfen
	if($buchung["description"] != $description) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "description=?";
		$args[] = $description;
	}
	
	// Kategorie prüfen
	if($edit_connected) {
		if($buchung["kategorie"] != $kategorie) {
			// Prüfen ob die Kategorie existiert und zu dem User gehört
			if(!empty($kategorie) && is_numeric($kategorie) && $kategorie != 0) {
				if(!kategorieValid($kategorie)) {
					return array("status" => "error", "message" => "Diese Kategorie existiert nicht oder gehört nicht zu diesem User");
				}
			}
			if(strlen($newVals) > 0) {
				$newVals .= ", ";
			}
			$newVals .= "kategorie=?";
			$args[] = $kategorie;
		}
	}
	
	// Datum prüfen
	if($buchung["datum"] != $datum) {
		if(getMonthFromDate($datum) != getMonthFromDate($buchung["datum"])) {
			// Evtl abgeschlossene Monate seit der Buchung löschen damit sie neu berechnet werden
			$sql_dm = "DELETE FROM *PREFIX*md_fibu_months WHERE konto='".$buchung["konto"]."' AND month>='".getMonthFromDate($buchung["datum"])."' OR month>='".getMonthFromDate($datum)."'";
			$res_dm = query($sql_dm);
			if(isSqlError($res_dm)) {
				return array("status" => "error", "message" => "Fehler beim auflösen des abgeschlossenen Monats:" . getSqlError());
			}
		}
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "datum=?";
		$args[] = $datum;
	}
	
	// Zeit prüfen
	if($buchung["zeit"] != $zeit) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "zeit=?";
		$args[] = $zeit;
	}
	
	// Wurde etwas geändert?
	if(strlen($newVals) == 0) {
		return array("status" => "error", "message" => "Keine Änderungen zum Speichern");
	}
	
	// Änderungen speichern
	$sql_u = "UPDATE *PREFIX*md_fibu_buchungen SET ".$newVals." WHERE id='".$buchung["id"]."'";
	$res_u = query($sql_u, $args);
	if(isSqlError($res_u)) {
		return array("status" => "error", "message" => "Fehler beim Speichern: " . getSqlError());
	}

	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function deleteBuchung($id, $del_connected = true) {
	// Übergaben prüfen
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Buchungs-ID übergeben: " . $id);
	}
	if($del_connected && !buchungValid($id)) {
		return array("status" => "error", "message" => "Diese Buchung existiert nicht oder gehört zu einem anderen User");
	}
	
	$id = floor($id);
	$sql_b = "SELECT * FROM *PREFIX*md_fibu_buchungen WHERE id='".$id."' LIMIT 0,1";
	$res_b = query($sql_b);
	$buchung = $res_b->fetchRow();

	// Liegt die Buchung in einem anderen Monat?
	if(getMonthFromDate($buchung["datum"]) != getThisMonth()) {
		// Evtl abgeschlossene Monate seit der Buchung löschen damit sie neu berechnet werden
		$sql_dm = "DELETE FROM *PREFIX*md_fibu_months WHERE konto=? AND month>=?";
		$res_dm = query($sql_dm, array($buchung["konto"], getMonthFromDate($buchung["datum"])));
		if(isSqlError($res_dm)) {
			return array("status" => "error", "message" => "Fehler beim auflösen des abgeschlossenen Monats: " . getSqlError());
		}
	}

	// Verweise löschen
	$result = deleteConnection($id, $del_connected);
	if($result["status"] == "error") {
		return array("status" => "error", "message" => "Fehler beim Löschen des Verweises: " . $result["message"]);
		
	}
	
	if($del_connected) {
		// Prüfen ob wir auf einem geteilten Konto sind
		$sql_cs = "SELECT * FROM *PREFIX*md_fibu_share_connection WHERE buchung_1=? OR buchung_2=? LIMIT 0,1";
		$res_cs = query($sql_cs, array($id, $id));
		if($res_cs->rowCount() > 0) {
			$share = $res_cs->fetchRow();
			// Die gegenläufige Buchung muss ebenfalls entfernt werden
			$b2 = ($share["buchung_1"] == $id) ? $share["buchung_2"] : $share["buchung_1"];
			$result2 = deleteBuchung($b2, false);
			if($result2["status"] == "error") {
				return array("status" => "error", "message" => "Fehler beim Löschen der gegenläufigen Buchung: " . $result2["message"]);
			} else {
				// Und Verknüpfung löschen
				$sql_ds = "DELETE FROM *PREFIX*md_fibu_share_connection WHERE buchung_1=? OR buchung_2=?";
				$res_ds = query($sql_ds, array($id, $id));
				if(isSqlError($res_ds)) {
					return array("status" => "error", "message" => "Fehler beim Löschen der Share-verbindung: " . getSqlError());
				}
			}
		}
	}
		
	// Buchung löschen
	$sql_db = "DELETE FROM *PREFIX*md_fibu_buchungen WHERE id=?";
	$res_db = query($sql_db, array($id));
	if(isSqlError($res_db)) {
		return array("status" => "error", "message" => "Fehler beim Löschen der Buchung: " . getSqlError());
	}

	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function getBuchungDetails($id) {
	$data = array();

	// Prüfen ob die ID gültig ist
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Buchungs-ID übergeben: " . $id);
	}

	$id = floor($id);

	// Prüfen ob die Buchung existiert und zu dem User gehört
	if(!buchungValid($id)) {
		return array("status" => "error", "message" => "Diese Buchung existiert nicht oder gehört nicht zu diesem User");
	}
	
	// Buchungsdetails laden
	$sql_b = "SELECT * FROM *PREFIX*md_fibu_buchungen WHERE id=? LIMIT 0,1";
	$res_b = query($sql_b, array($id));
	$buchung = $res_b->fetchRow();
	
	$data["id"] = $buchung["id"];
	$data["name"] = $buchung["name"];
	$data["description"] = $buchung["description"];
	$data["betrag"] = $buchung["betrag"] / 100;
	$data["datum"] = $buchung["datum"];
	$data["zeit"] = $buchung["zeit"];
	$data["kategorie"] = $buchung["kategorie"];
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "", "data" => $data);
}
?>