<?php
function kategorieValid($kategorie_id) {
	// Prüfen ob die Kategorie existiert und zu dem aktuellen User gehört
	$sql_cka = "SELECT 1 FROM *PREFIX*md_fibu_kategorien WHERE uid=? AND id=?";
	$res_cka = query($sql_cka, array(getUID(), $kategorie_id));
	if($res_cka->rowCount() != 1) {
		return false;
	}
	return true;
}

function deleteKategorie($id) {
	// Prüfen ob die ID gültig ist
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige ID übergeben: " . $id);
	}

	$id = floor($id);

	// prüfen ob die Kategorie existiert und zu dem User gehört
	if(!kategorieValid($id)) {
		return array("status" => "error", "message" => "Diese Kategorie existiert nicht oder gehört einem anderen User");
	}
		
	// Alle Buchungen der Kategorie auf 0 setzen
	$sql_b = "UPDATE *PREFIX*md_fibu_buchungen SET kategorie='0' WHERE kategorie=?";
	$res_b = query($sql_b, array($id));
	if(isSqlError($res_b)) {
		return array("status" => "error", "message" => "Fehler beim Zurücksetzen der Kategorie in den Buchungen: " . getSqlError());
	}
	
	// Kategorie löschen
	$sql_d = "DELETE FROM *PREFIX*md_fibu_kategorien WHERE id=?";
	$res_d = query($sql_d, array($id));
	if(isSqlError($res_d)) {
		return array("status" => "error", "message" => "Fehler beim Löschen der Kategorie: " . getSqlError());
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function getKategorien() {
	$data = array();
	$index = 0;

	// Alle Kategorien des aktuellen Users laden
	$sql_k = "SELECT * FROM *PREFIX*md_fibu_kategorien WHERE uid=?";
	$res_k = query($sql_k, array(getUID()));
	while($kategorie = $res_k->fetchRow()) {
		
		$data[$index] = array();
		$data[$index]["id"] = $kategorie["id"];
		$data[$index]["name"] = $kategorie["name"];
		$data[$index]["ignore"] = $kategorie["ignore"];
		
		$index++;
	}
	
	// Rückgabe der Daten
	return array("status" => "success", "message" => "", "data" => $data);
}

function editKategorie($id, $ignore, $name) {
	// Prüfen der Eingaben
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Kategorie-ID: " . $id);
	}
	if(!in_array($ignore, array(0,1))) {
		return array("status" => "error", "message" => "Ungültiges Ignore-Kennzeichen: " . $ignore);
	}
	if(empty($name)) {
		return array("status" => "error", "message" => "Kein Name übergeben");
	}

	$id = floor($id);
	$ignore = floor($ignore);

	// Prüfen ob die Kategorie existiert und dem User gehört
	if(!kategorieValid($id)) {
		return array("status" => "error", "message" => "Diese Kategorie existiert nicht oder gehört zu einem anderen User");
	}
	
	// Daten der Kategorie laden
	$sql_k = "SELECT * FROM *PREFIX*md_fibu_kategorien WHERE id=? LIMIT 0,1";
	$res_k = query($sql_k, array($id));
	if(isSqlError($res_k)) {
		return array("status" => "error", "message" => "Fehler beim Laden der Kategorie-Daten: " . getSqlError());
	}
	
	$kategorie = $res_k->fetchRow();
	$newVals = "";
	$args = array();
	
	// Änderung am Name prüfen
	if($name != $kategorie['name']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "name=?";
		$args[] = $name;
	}
	
	// Änderung am Ignore-Kennzeichen prüfen
	if($ignore != $kategorie['ignore']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "`ignore`='".(($ignore == 1)?"1":"0")."'";
	}
	
	// Liegen Änderungen vor?
	if(strlen($newVals) == 0) {
		return array("status" => "error", "message" => "Es wurde nichts geändert");
	}
		
	// Update durchführen
	$args[] = $id;
	$sql_u = "UPDATE *PREFIX*md_fibu_kategorien SET ".$newVals." WHERE id=?";
	$res_u = query($sql_u, $args);
	if(\OCP\DB::isError($res_u)) {
		return array("status" => "error", "message" => "Fehler beim Speichern der Änderungen: " . getSqlError());
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function newKategorie($name, $ignore) {
	// Eingaben prüfen
	if(empty($name)) {
		return array("status" => "error", "message" => "Kein Name übergeben");
	}
	if(!in_array($ignore, array(0,1))) {
		return array("status" => "error", "message" => "Ungültiges Ignore-Kennzeichen übergeben: " . $ignore);
	}
		
	// Kategorie speichern
	$sql_i = "INSERT INTO *PREFIX*md_fibu_kategorien (`uid`, `name`, `ignore`) VALUES (?, ?, ?)";
	$res_i = query($sql_i, array(getUID(), $name, $ignore));
	if(isSqlError($res_i)) {
		return array("status" => "error", "message" => "Fehler beim Speichern: " . getSqlError());
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}
?>