<?php
function kontoValid($konto_id) {
	// Prüfen ob das Konto existiert und zu dem User gehört
	$sql_cko = "SELECT 1 FROM *PREFIX*md_fibu_konten WHERE uid=? AND id=?";
	$res_cko = query($sql_cko, array(getUID(), $konto_id));
	if($res_cko->rowCount() != 1) {
		return false;
	}
	return true;
}

function getNextKontoPosition($uid) {
	// Maximale Position + 1 der Konten des aktuellen Users ermitteln
	$sql_c = "SELECT MAX(position) + 1 AS nextPos FROM *PREFIX*md_fibu_konten WHERE uid='".$uid."'";
	$res_c = query($sql_c);
	if($res_c->rowCount() > 0) {
		$row = $res_c->fetchRow();
		return ($row['nextPos'] != null) ? $row['nextPos'] : 0;
	}
	// Noch keine Konten vorhanden? Dann ist die nächste Position 0
	return 0;
}

function deleteKonto($id) {
	// Prüfen ob die ID gültig ist
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Konto-ID übergeben: " . $id);
	}

	$id = floor($id);

	// Prüfen ob das Konto existiert und zu dem User gehört
	if(!kontoValid($id)) {
		return array("status" => "error", "message" => "Dieses Konto existiert nicht oder gehört einem anderen User");
	}
		
	// Alle Buchungen des Kontos löschen
	$sql_b = "SELECT id FROM *PREFIX*md_fibu_buchungen WHERE konto=?";
	$res_b = query($sql_b, array($id));
	while($row = $res_b->fetchRow()) {
		$result = deleteBuchung($row['id']);
		if($result["status"] == "error") {
			return array("status" => "error", "message" => "Fehler beim Löschen einer Buchung: " . $result["message"]);
		}
	}
	
	// Alle Monatssalden löschen
	$sql_dm = "DELETE FROM *PREFIX*md_fibu_months WHERE konto=?";
	$res_dm = query($sql_dm, array($id));
	if(isSqlError($res_dm)) {
		return array("status" => "error", "message" => "Fehler beim Auflösen der abgeschlossenen Monate: " . getSqlError());
	}
	
	// Ausleihe-Verknüpfung löschen
	$sql_dl = "DELETE FROM *PREFIX*md_fibu_share WHERE konto_1=? OR konto_2=?";
	$res_dl = query($sql_dl, array($id, $id));
	if(isSqlError($res_dl)) {
		return array("status" => "error", "message" => "Fehler beim Löschen der Ausleihe-Verbindung: " . getSqlError());
	} 
	
	// Konto löschen
	$sql_d = "DELETE FROM *PREFIX*md_fibu_konten WHERE id=?";
	$res_d = query($sql_d, array($id));
	if(isSqlError($res_d)) {
		return array("status" => "error", "message" => "Fehler beim Löschen des Kontos: " . getSqlError());
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function getKonto($id) {
	
	$data = array();
	
	// Prüfen ob die ID gültig ist
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return array("status" => "error", "message" => "Ungültige Konto-ID:" . $id);
	}

	$id = floor($id);

	// Prüfen ob das Konto existiert und zu dem User gehört
	if(!kontoValid($id)) {
		return array("status" => "error", "message" => "Dieses Konto existiert nicht oder gehört zu einem anderen User");
	} 
		
	// Kontodaten laden
	$sql_k = "SELECT * FROM *PREFIX*md_fibu_konten WHERE id=? LIMIT 0,1";
	$res_k = query($sql_k, array($id));
	if(isSqlError($res_k)) {
		return array("status" => "error", "message" => "Fehler beim Laden der Kontodaten: " . getSqlError());
	}

	$konto = $res_k->fetchRow();
	$data["id"] = $id;
	$data["name"] = $konto['name'];
	$data["info"] = $konto['info'];
	$data["position"] = $konto['position'];
	$data["ignore"] = $konto['ignore'];
	
	
	// Rückgabe der Daten
	return array("status" => "success", "message" => "", "data" => $data);
}

function getKonten() {
	$data = array();
	$data["gesamt"] = gesamt();
	$data["konten"] = array();

	// Alle Konten des aktuellen Users laden
	$sql = "SELECT * FROM *PREFIX*md_fibu_konten WHERE uid=? ORDER BY position ASC";
	$result = query($sql, array(getUID()));
	while($konto = $result->fetchRow()) {

		$index = count($data["konten"]);
		$data["konten"][$index] = array();
		$data["konten"][$index]["name"] = $konto['name'];
		$data["konten"][$index]["id"] = $konto['id'];
		$data["konten"][$index]["saldo"] = kontostand($konto['id']);

	}
	
	// Rückgabe der Daten
	return array("status" => "success", "message" => "", "data" => $data);
}

function getKontoDetails($konto, $year, $month) {

	$data = array();

	// Eingaben prüfen
	if(empty($konto) || !is_numeric($konto) || $konto == 0) {
		return array("status" => "error", "message" => "Ungültige Konto-ID: " . $konto);
	}
	if(empty($year) || !is_numeric($year) || $year < 1) {
		return array("status" => "error", "message" => "Ungültiges Jahr: " . $year);
	}
	if(empty($month) || !is_numeric($month) || $month < 1 || $month > 12) {
		return array("status" => "error", "message" => "Ungültiger Monat: " . $month);
	}

	$konto = floor($konto);
	$year = floor($year);
	$month = floor($month);

	// Prüfen ob das Konto existiert und zu dem User gehört
	if(!kontoValid($konto)) {
		return array("status" => "error", "message" => "Dieses Konto existiert nicht oder gehört zu einem anderen User");
	}
	
	$data["konto"] = $konto;
	$data["year"] = $year;
	$data["month"] = $month;

	$data["buchungen"] = array();

	// Datumsgrenzen festlegen
	$date_from = str_pad($year, 4, "0", STR_PAD_LEFT)."-".str_pad($month, 2, "0", STR_PAD_LEFT)."-01";
	$date_to = addMonths($date_from, 1);
	
	// Alle Buchungen auf dem Konto innerhalb der Grenzen laden
	$sql_b = "SELECT b.*, k.name AS kategorie_name FROM *PREFIX*md_fibu_buchungen b LEFT JOIN *PREFIX*md_fibu_kategorien k ON b.kategorie=k.id WHERE b.konto=? AND b.datum>=? AND b.datum<? ORDER BY b.datum ASC, b.zeit ASC";
	$result_b = query($sql_b, array($konto, $date_from, $date_to));
	while($buchung = $result_b->fetchRow()) {
	
		$index = count($data["buchungen"]);
		$data["buchungen"][$index] = array();
		
		$data["buchungen"][$index]["id"] = $buchung["id"];
		$data["buchungen"][$index]["name"] = $buchung["name"];
		$data["buchungen"][$index]["description"] = $buchung["description"];
		$data["buchungen"][$index]["zeit"] = $buchung["zeit"];
		$data["buchungen"][$index]["datum"] = $buchung["datum"];
		$data["buchungen"][$index]["kategorie"] = $buchung["kategorie"];
		$data["buchungen"][$index]["kategorie_name"] = (!empty($buchung["kategorie_name"]) ? $buchung["kategorie_name"] : "Keine Kategorie");
		$data["buchungen"][$index]["betrag"] = $buchung["betrag"] / 100;
		$data["buchungen"][$index]["created_by"] = ($buchung["created_by"] != getUID() ? $buchung["created_by"] : "");
		
		$data["buchungen"][$index]["connection"] = array();
		
		// Prüfen ob ein Verweis vorhanden ist
		$sql_c = "SELECT * FROM *PREFIX*md_fibu_connection WHERE buchung_pos=? OR buchung_neg=? LIMIT 0,1";
		$res_c = query($sql_c, array($buchung['id'], $buchung['id']));
		if($res_c->rowCount() == 1) {
			$con = $res_c->fetchRow();
			
			// Details der verknüpften Buchung laden
			$sql_b2 = "SELECT b.*, k.name AS konto_name FROM *PREFIX*md_fibu_buchungen b, *PREFIX*md_fibu_konten k WHERE b.konto=k.id AND b.id=? LIMIT 0,1";
			$res_b2 = query($sql_b2, array(($con["buchung_pos"] == $buchung["id"]) ? $con["buchung_neg"] : $con["buchung_pos"]));
			if($res_b2->rowCount() == 1) {
				$b2 = $res_b2->fetchRow();
				$data["buchungen"][$index]["connection"]["id"] = $b2["id"];
				$data["buchungen"][$index]["connection"]["konto"] = $b2["konto"];
				$data["buchungen"][$index]["connection"]["year"] = substr($b2["datum"], 0, 4);
				$data["buchungen"][$index]["connection"]["month"] = substr($b2["datum"], 5, 2);
				$data["buchungen"][$index]["connection"]["konto_name"] = $b2["konto_name"];
			}
		}
		
	}
	
	// Vormonats-Saldo anfügen
	$data["vormonat"] = kontostandMonth($konto, addMonths($date_from, -1));
	
	// Rückgabe der Daten
	return array("status" => "success", "message" => "", "data" => $data);
}

function editKonto($id, $name, $info, $position, $ignore) {
	// Eingaben prüfen
	if(empty($id) || !is_numeric($id)) {
		return array("status" => "error", "message" => "Ungültige Konto-ID: " . $id);
	}
	if(empty($position) || !is_numeric($position)) {
		return array("status" => "error", "message" => "Ungültige Position: " . $position);
	}
	if(empty($name)) {
		return array("status" => "error", "message" => "Kein Name übergeben");
	}
	if(!in_array($ignore, array(0, 1))) {
		return array("status" => "error", "message" => "Ungültiges Ignore-Kennzeichen: " . $ignore);
	}

	$id = floor($id);
	$position = floor($position);

	// Prüfen ob das Konto existiert und zu dem User gehört
	if(!kontoValid($id)) {
		return array("status" => "error", "message" => "Dieses Konto existiert nicht oder gehört zu einem anderen User");
	} 
	
	// Kontodaten laden
	$sql_k = "SELECT * FROM *PREFIX*md_fibu_konten WHERE id=? LIMIT 0,1";
	$res_k = query($sql_k, array($id));
	if(isSqlError($res_k)) {
		return array("status" => "error", "message" => "Fehler beim Laden der Kontodaten: " . getSqlError());
	}
	
	$konto = $res_k->fetchRow();
	$newVals = "";
	$args = array();
	
	// Änderung am Name prüfen
	if($name != $konto['name']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "name=?";
		$args[] = $name;
	}
	
	// Änderung an Info prüfen
	if($info != $konto['info']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "info=?";
		$args[] = $info;
	}
	
	// Änderung an Position prüfen
	if($position != $konto['position']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "position=?";
		$args[] = $position;
	}
	
	// Änderung an Ignore prüfen
	if($ignore != $konto['ignore']) {
		if(strlen($newVals) > 0) {
			$newVals .= ", ";
		}
		$newVals .= "`ignore`=?";
		$args[] = $ignore;
	}
	
	// Liegen Änderungen vor?
	if(strlen($newVals) == 0) {
		$status = "error";
		$message = "Es wurde nichts geaendert";
	}
	
	// Änderungen speichern
	$args[] = $id;
	$sql_u = "UPDATE *PREFIX*md_fibu_konten SET ".$newVals." WHERE id=?";
	$res_u = query($sql_u, $args);
	if(isSqlError($res_u)) {
		return array("status" => "error", "message" => "Fehler beim Speichern: " . getSqlError());
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}

function newKonto($name, $info, $share_user, $inverted, $ignore) {
	// Eingaben prüfen
	if(empty($name)) {
		return array("status" => "error", "message" => "Kein Name übergeben");
	}
	if($share_user == getUID()) {
		return array("status" => "error", "message" => "Ungültige User-ID für geteiltes Konto: " . $share_user);
	}
	if(!in_array($inverted, array(0, 1))) {
		return array("status" => "error", "message" => "Ungültiges Inverted-Kennzeichen: " . $inverted);
	}
	if(!in_array($ignore, array(0, 1))) {
		return array("status" => "error", "message" => "Ungültiges Ignore-Kennzeichen: " . $ignore);
	}
	if(empty($info)) {
		$info = "";
	}

	// Wenn ein Geteilter Benutzer übergeben wurde, prüfen ob dieser existiert
	if(!empty($share_user)) {
		$sql_cu = "SELECT 1 FROM *PREFIX*users WHERE uid=? LIMIT 0,1";
		$res_cu = query($sql_cu, array($share_user));
		if($res_cu->rowCount() != 1) {
			return array("status" => "error", "message" => "Nicht existierende User-ID für geteiltes Konto");
		}
	}
	
	// Konto anlegen
	$sql_i = "INSERT INTO *PREFIX*md_fibu_konten (`uid`, `name`, `info`, `position`, `ignore`) VALUES (?, ?, ?, ?, ?)";
	$res_i = query($sql_i, array(getUID(), $name, $info, getNextKontoPosition(getUID()), $ignore));
	if(isSqlError($res_i)) {
		return array("status" => "error", "message" => "Fehler beim Speichern des Kontos: " . getSqlError());
	}
		$id = sqlInsertId();
		
	// Wurde ein Geteilter User übergeben?
	if(!empty($share_user)) {
		// Konto für den anderen User anlegen
		$share_name = $inverted ? "Ausleihe ".getUID() : "Geteilt mit ".getUID();
		$sql_i2 = "INSERT INTO *PREFIX*md_fibu_konten (`uid`, `name`, `info`, `position`, `ignore`) VALUES (?, ?, '', ?, ?)";
		$res_i2 = query($sql_i2, array($share_user, $share_name, getNextKontoPosition($share_name), $ignore));
		if(isSqlError($res_i2)) {
			return array("status" => "error", "message" => "Fehler beim Speichern des zweiten Kontos: " . getSqlError());
		}
		
		$id2 = sqlInsertId();
		
		// Konten miteinander verbinden
		$sql_il = "INSERT INTO *PREFIX*md_fibu_share (konto_1, konto_2, inverted) VALUES(?, ?, ?)";
		$res_il = query($sql_il, array($id, $id2, $inverted));
		if(isSqlError($res_il)) {
			return array("status" => "error", "message" => "Fehler beim Speichern der Geteilten Verbindung: " . getSqlError());
		}
	}
	
	// Rückgabe alles OK
	return array("status" => "success", "message" => "");
}
?>