<?php
// Aktuellen Kontostand eines Kontos ermitteln
function kontostand($id) {
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return 0;
	}
	$id = floor($id);
	return kontostandLastMonth($id) + sumThisMonth($id);
}

// Abschluss-Betrag des letzten Monats ermitteln
function kontostandLastMonth($id) {
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return 0;
	}
	$id = floor($id);
	return kontostandMonth($id, getLastMonth());
}

// Abschluss-Betrag eines Kontos in einem bestimmten Monat ermitteln
function kontostandMonth($id, $month) {
	if(!dateValid($month) || empty($id) || !is_numeric($id) || $id == 0) {
		return 0;
	}
	$id = floor($id);
	$vormonat = schlussstandMonth($id, addMonths($month, -1));
	$aktuell = buchungsbetragMonth($id, $month);
	return $vormonat + $aktuell;
}

// Summe aller Buchungen auf einem Konto im aktuellen Monat ermitteln
function sumThisMonth($id) {
	if(empty($id) || !is_numeric($id) || $id == 0) {
		return 0;
	}
	$id = floor($id);
	$sql = "SELECT SUM(betrag) AS betrag FROM *PREFIX*md_fibu_buchungen WHERE konto=? AND datum>=?";
	$result = query($sql, array($id, getThisMonth()));
	$betrag = 0;
	while($row = $result->fetchRow()) {
		$betrag += $row['betrag'] / 100;
	}
	return $betrag;
}

// Kontostand aller Konten ermitteln
function gesamt() {
	$sql = "SELECT id FROM *PREFIX*md_fibu_konten WHERE uid=? AND `ignore`=0";
	$result = query($sql, array(getUID()));
	$summe = 0;
	while($row = $result->fetchRow()) {
		$summe += kontostand($row['id']);
	}
	return $summe;
}

// Abschluss-Betrag eines Kontos in einem bestimmten Monat ermitteln
function schlussstandMonth($konto_id, $monat) {
	if(!dateValid($monat) || empty($konto_id) || !is_numeric($konto_id) || $konto_id == 0) {
		return 0;
	}
	$konto_id = floor($konto_id);
	$vormonat = 0;
	$sqlM = "SELECT betrag FROM *PREFIX*md_fibu_months WHERE konto=? AND month=? ORDER BY month DESC LIMIT 0,1";
	$resM = query($sqlM, array($konto_id, $monat));
	if($resM->rowCount() == 1) {
		$month = $resM->fetchRow();
		$vormonat = $month['betrag'] / 100;
	}
	return $vormonat;
}

// Summe aller Buchungen in einem bestimmten Monat ermitteln
function buchungsbetragMonth($konto_id, $monat) {
	if(!dateValid($monat) || empty($konto_id) || !is_numeric($konto_id) || $konto_id == 0) {
		return 0;
	}
	$konto_id = floor($konto_id);
	$aktuell = 0;
	$sqlB = "SELECT SUM(betrag) AS betrag FROM *PREFIX*md_fibu_buchungen WHERE konto=? AND datum>=? AND datum<?";
	$resB = query($sqlB, array($konto_id, $monat, addMonths($monat, 1)));
	if($resB->rowCount() == 1) {
		$buchungen = $resB->fetchRow();
		$aktuell = $buchungen['betrag'] / 100;
	}
	return $aktuell;
}
?>